import java.util.*;

class BinaryTree { 

    /////////////////////////////
    // main method/test

    public static void main(String[] args) { 

        // Construct a little tree
        BinaryTree b = new BinaryTree();
        b.insert(840); // root node
        b.insert(5); b.insert(11); // left and right children
        b.insert(8); b.insert(3); b.insert(82); b.insert(35); // etc.

        // Test out our search method
        System.out.print( "Searching for 5: " );
        System.out.println( b.search(35) );

        // Test out our sort method
        b.sort();
        System.out.println(b);
        System.out.printf("To confirm sorted, confirm %d < %d \n", b.head.left.data, b.head.right.data );
        System.out.printf("To confirm sorted, confirm %d < %d \n", b.head.left.left.data, b.head.left.right.data );
        System.out.printf("To confirm sorted, confirm %d < %d \n", b.head.right.left.data, b.head.right.right.data );
    }


    ////////////////////////////////
    // Binary Tree class 

    Node head;

    /** Default constructor */ 
    public BinaryTree() {
        this.head = new Node(0);
    }

    /** Constructor passing a Node */
    public BinaryTree(int z) {
        this.head = new Node(z);
    }

    /** Insert using breadth-first traversal */
    public void insert(int z) { 
        LinkedList<Node> q = new LinkedList<Node>();
        q.addLast(this.head);
        while(q.size()>0) {
            Node next = q.pop();
            if(next.left==null) {
                next.left = new Node(z);
            } else if(next.right==null) {
                next.right = new Node(z);
            } else {
                q.addLast(next.left);
                q.addLast(next.right);
            }
        }
    }


    /** Public sort method */
    public void sort() {
        // Sorts the binary tree: left < right
        sort(this.head);
    }

    /** Private recursive sort method */
    private void sort(Node n) {
        // We require node and its children to be non-null 
        if( n != null && n.left != null && n.right != null ) { 
            // If left and right are out of order, set them aright
            if( n.left.data >= n.right.data ) { 
                Node temp = n.left;
                n.left = n.right;
                n.right = temp;
            }
            // No matter what, we need to sort the children's children
            sort(n.left);
            sort(n.right);
        }
        // When wer'e all done, we're all done.
        return;
    }

    /** Public search method */
    public boolean search(int value) {
        // searches for value in binary tree
        return search(value,this.head);
    }

    /** Private recursive search method */
    private boolean search(int value, Node n) {
        if( n == null ) { 
            return false;
        } else if( n.data == value ) { 
            return true;
        } else {
            return search(value, n.left) || search(value, n.right);
        }
    }
}

class Node { 
    int data;
    Node left;
    Node right;

    public Node(int data) { 
        this(data,null,null);
    }

    public Node(int data, Node left, Node right) { 
        this.data = data;
        this.left = left;
        this.right = right;
    }
}
