import java.util.*;

/** 
 * LList1 - Round 1 of linked list implementation.
 *
 * This implementation of a linked list focuses on basic functionality:
 * constructor, toString(), add(value), add(index,value), remove(index), indexOf(value)
 */

public class LList1 {

	public static void main(String[] args) { 

		LList1 list = new LList();

		Random r = new Random();
		for(int i=0; i<1000; i++) { 
			list.add( r.nextInt() );
		}
		System.out.println(list.toString());

	}





	private Node head;

	public LList1() {
		this.head = null;
	}

	// Print it out!
	public String toString() { 
		String str = "";
		Node current = head;
		while(current != null ) {
			str += current.data;
			if(current.next != null) {
				str += "->";
			}
			current = current.next;
		}
		return str;
	}


	// Add an item to the list.
	// This starts with a pointer to the beginning of the list,
	// and runs all the way through the list to the end. 
	public void add(int z) {
		if(this.head==null) { 
			head = new Node(z); 
		} else {
			Node iter = this.head;
			while(iter.next != null) { 
				iter = iter.next;
			}
			// Now we are at the end...
			iter.next = new Node(z);
		}
	}

	// Add an item to a location in the list.
	// This starts with a pointer to the beginning of the list,
	// and runs all the way up to index. It then swaps out (carefully)
	// the pointers in the list.
	public void add(int index, int z) {
		if(index==0) { 
			// New head node equal data (z) and pointer to old head node
			this.head = new Node(z, this.head);
		} else {
			// Get the node right in front of the one we are going to add
			Node pointer = nodeAt(index-1);

			// Add the new node right in front
			// New pointer.next equals data (z) and old pointer.next
			pointer.next = new Node(z, pointer.next);
		}
	} 

	// Remove an index from a list.
	public void remove(int index) { 
		if( index==0 ) { 
			head = head.next;
		} else {
			// Get all the way up to the node just in front of the node we are removing
			Node current = nodeAt(index-1);

			// Now blow away any reference to the next node
			current.next = current.next.next;
		}
	}


	public int indexOf(int value) { 
		int index = 0;
		Node current = head;
		while(current != null) { 
			if(current.data == value) {
				return index;
			}
			index++;
			current = current.next;
		}
		return -1;
	}


	// Get the node at index i
    private Node nodeAt(int index) {
        Node pointer = head;
        for (int i = 0; i < index; i++) {
            pointer = pointer.next;
        }
        return pointer;
    }

}

class Node {

	// A Linked List Node is just a data burrito. 

    public int data;       // data stored in this node
	public Node next;      // llink to next node

	// default constructor: zero data, null link
	public Node() {
		this(0,null);
	}

    // constructor with data and a null link
    public Node(int data) {
        this(data, null);
    }

	// constructor with data and link
	public Node(int init, Node node0) {
		this.data = init;
		this.next = node0;
	}

}

