import java.util.*;

/**
 * LList3 - Round 3 of linked list implementation.
 *
 * This implementation adds a few recursive methods.
 *
 */
public class LList3 { 


	public static void main(String[] args) { 

		LList3 list = new LList3();

		Random r = new Random();
		for(int i=0; i<1000; i++) { 
			list.recursiveAdd( r.nextInt() );
		}
		list.recursiveAdd(5005);
		list.recursiveAdd(5005);
		list.recursiveAdd(5005);
		list.recursiveAdd(5005);
		list.recursiveAdd(5005);
		list.recursiveAdd(5005);
		list.recursiveAdd(5005);
		list.recursiveAdd(10484835);
		list.recursiveAdd(82349749);
		list.recursiveAdd(68498287);

		System.out.println(list.toString());

		list.removeDupesNoBuffer();

		System.out.println(list.toString());

	}




	private Node head;

	public LList3() { 
		this.head = null;
	}


	//////////////////////////////
	// Round 2

	public void removeDupes() { 
		if( this.head == null || this.head.next == null ) { 
			return;
		} 

		// values buffer
		Set<Integer> values = new TreeSet<Integer>();

		Node current = this.head;
		values.add(current.data);

		while( current.next != null ) {
			if( values.contains(current.next.data) ) {
				// remove the dupe. 
				// don't update current!
				current.next = current.next.next;
			} else {
				// only update current if we don't find a dupe.
				current = current.next;
			}
		}
	}

	public void removeDupesNoBuffer() { 
		if( this.head == null || this.head.next == null ) { 
			return;
		} 
		Node current = head;
		while(current != null ) {
			Node runner = current;
			while( runner.next != null ) { 
				if( current.data == runner.next.data ) {
					// remove the dupe. 
					// don't update current!
					runner.next = runner.next.next;
				} else {
					// only update current if we don't find a dupe.
					runner = runner.next;
				}
			}
			current = current.next;
		}
	}



	////////////////////////////
	// Round 1 

	// Print it out!
	public String toString() { 
		String str = "";
		Node current = head;
		while(current != null ) {
			str += current.data;
			if(current.next != null) {
				str += "->";
			}
			current = current.next;
		}
		return str;
	}


	// Add an item to the list.
	// This starts with a pointer to the beginning of the list,
	// and runs all the way through the list to the end. 
	public void add(int z) {
		if(this.head==null) { 
			head = new Node(z); 
		} else {
			Node iter = this.head;
			while(iter.next != null) { 
				iter = iter.next;
			}
			// Now we are at the end...
			iter.next = new Node(z);
		}
	}

	// Add an item to the list... recursively.
	// This uses a public non-recursive method that calls a private recursive method.
	public void recursiveAdd(int z) { 
		Node current = this.head;
		if(this.head==null) { 
			this.head = new Node(z);
		} else {
			recursiveAdd(current,z);
		}
	}

	// Recursive method to add value z to list. If current.next is null, we are at end of list; otherwise, calls itself with current.next.
	private void recursiveAdd(Node current, int z) { 
		if(current.next==null) { 
			current.next = new Node(z);
		} else {
			recursiveAdd(current.next,z);
		}
	}


	// Add an item to a location in the list.
	// This starts with a pointer to the beginning of the list,
	// and runs all the way up to index. It then swaps out (carefully)
	// the pointers in the list.
	public void add(int index, int z) {
		if(index==0) { 
			// New head node equal data (z) and pointer to old head node
			this.head = new Node(z, this.head);
		} else {
			// Get the node right in front of the one we are going to add
			Node pointer = nodeAt(index-1);

			// Add the new node right in front
			// New pointer.next equals data (z) and old pointer.next
			pointer.next = new Node(z, pointer.next);
		}
	} 

	// Remove an index from a list.
	public void remove(int index) { 
		if( index==0 ) { 
			head = head.next;
		} else {
			// Get all the way up to the node just in front of the node we are removing
			Node current = nodeAt(index-1);

			// Now blow away any reference to the next node
			current.next = current.next.next;
		}
	}


	// Get the index of value. If value is not in the list, return -1.
	public int indexOf(int value) { 
		int index = 0;
		Node current = head;
		while(current != null) { 
			if(current.data == value) {
				return index;
			}
			index++;
			current = current.next;
		}
		return -1;
	}


	// Get the node at index i
    private Node nodeAt(int index) {
        Node pointer = head;
        for (int i = 0; i < index; i++) {
            pointer = pointer.next;
        }
        return pointer;
    }

}

class Node {

	// A Linked List Node is just a data burrito. 

    public int data;       // data stored in this node
	public Node next;      // llink to next node

	// default constructor: zero data, null link
	public Node() {
		this(0,null);
	}

    // constructor with data and a null link
    public Node(int data) {
        this(data, null);
    }

	// constructor with data and link
	public Node(int init, Node node0) {
		this.data = init;
		this.next = node0;
	}

}

