/**
 * Templated Double-Ended Queue (Deque) Class.
 * 
 * Accepts aribtrary class type.
 */
public class TDeque<T> {
    Node<T> head;
    Node<T> tail;
    boolean empty;

    /////////////////////////////////////////////////////
    // main method
    //
    public static void main(String[] args) throws EmptyDequeException {

        TDeque<Integer> d = new TDeque<Integer>(null);

        System.out.println("Populating deque...");
        for(int i=1; i<=10; i++ ) {
            d.append(new Integer(i));
        }
        System.out.println("Done populating deque.");
        
        System.out.println("Popping from deque...");
        while(!d.isEmpty()){
            System.out.println(d.pop());
        }
        System.out.println("Done popping from deque.");

        System.out.println("Populating second deque...");
        for(int i=1; i<=10; i++ ) {
            d.appendLeft(new Integer(i));
        }
        System.out.println("Done populating second deque.");
        
        System.out.println("Popping from second deque...");
        while(!d.isEmpty()){
            System.out.println(d.popLeft());
        }
        System.out.println("Done popping from second deque.");

    }
    //
    /////////////////////////////////////////////////////

    public TDeque() {
        this(null);
    }

    public TDeque(Node<T> n) {
        this.head = n;
        this.tail = n;
        if(n==null) { 
            this.empty = true;
        } else {
            this.empty = false;
        }
    }

    public boolean isEmpty(){
        return this.empty;
    }

    public void append(T z) { 
        // New head of list should point to old head of list 
        Node<T> n = new Node<T>(z, this.head);
        // Break deque's link to old head of list, point to new head of list
        this.head = n;
        // If empty, not empty anymore. Update tail to point at head. 
        if(this.empty) {
            this.tail = this.head;
            this.empty = false;
        }
    }
    
    public void appendLeft(T z) { 
        // New tail of list should point to null
        Node<T> n = new Node<T>(z);
        // Get pointer to old tail (may be null)
        Node<T> old_tail = this.tail;
        // Break deque's link to old tail of list, point to new tail of list
        this.tail = n;
        // If empty, not empty anymore. Update head to point to tail.
        if(this.empty) { 
            this.head = this.tail;
            this.empty = false;
        } else {
            // Old tail pointer is not null, b/c list was not empty. 
            // Update its next pointer.
            old_tail.next = this.tail;
        }
    }

    public T pop() throws EmptyDequeException {
        if(this.empty) {
            throw new EmptyDequeException("Error: Empty deque object! Use the empty() method.");
        }
        // Get pointer to old head of list
        Node<T> n = this.head;
        // Break deque's link to old head of list, point to new head of list
        if(this.head.next != null ) { 
            this.head = n.next;
        } else {
            this.head = null;
            this.tail = null;
            this.empty = true;
        }
        // Return the old head of the list 
        return n.data;
    }

    public T popLeft() throws EmptyDequeException {
        if(this.empty) { 
            throw new EmptyDequeException("Error: Empty deque object! Use the empty() method.");
        }
        // Get pointer to old tail of list
        Node<T> old_tail = this.tail;
        // Get pointer to new tail of list 
        Node<T> runner = this.head;
        if( runner.equals(this.tail) ) {
            this.head = null;
            this.tail = null;
            this.empty = true;
            return runner.data;
        } 
        while( runner.next != this.tail ) {
            runner = runner.next;
        }
        // Break deque's link to old tail of list, point to new tail of list
        this.tail = runner;
        runner.next = null;
        // Return the old tail of the list
        return old_tail.data;
    }
}

class Node<T> {
    T data; 
    Node<T> next;
    public Node(T the_data) {
        this(the_data,null);
    }

    public Node(T the_data, Node<T> the_next) { 
        this.data = the_data;
        this.next = the_next;
    }
}

class EmptyDequeException extends Exception { 
    public EmptyDequeException(String message){
        super(message);
    }
}

