import java.util.*;

public class TList<T> { 

	public static void main(String[] args) { 

        TList<Point> t = new TList<Point>();
        for(int i=0; i<5; i++) {
            Point p = new Point(i,i);
            t.recursiveAdd(p);
        }
		System.out.println(t.toString());

	}

	private Node<T> head;

	public TList() { 
		this(null);
	}

    public TList(Node<T> the_head) {
        this.head = the_head;
    }




	////////////////////////////
	// Round 1 

	/// Print it out!
	public String toString() { 
		String str = "";
		Node<T> current = head;
		while(current != null ) {
			str += current.data.toString();
			if(current.next != null) {
				str += "->";
			}
			current = current.next;
		}
		return str;
	}


    /**
	 * Add an item to the list.
	 * This starts with a pointer to the beginning of the list,
	 * and runs all the way through the list to the end. 
     */
	public void add(T z) {
		if(this.head==null) { 
			head = new Node<T>(z); 
		} else {
			Node<T> iter = this.head;
			while(iter.next != null) { 
				iter = iter.next;
			}
			// Now we are at the end...
			iter.next = new Node<T>(z);
		}
	}

    /**
	  * Add an item to the list... recursively.
	  * This uses a public non-recursive method that calls a private recursive method.
      */
	public void recursiveAdd(T z) { 
		Node<T> current = this.head;
		if(this.head==null) { 
			this.head = new Node<T>(z);
		} else {
			recursiveAdd(current,z);
		}
	}

	/// Recursive method to add value z to list. If current.next is null, we are at end of list; otherwise, calls itself with current.next.
	private void recursiveAdd(Node<T> current, T z) { 
		if(current.next==null) { 
			current.next = new Node<T>(z);
		} else {
			recursiveAdd(current.next,z);
		}
	}


    /**
	  * Add an item to a location in the list.
	  * This starts with a pointer to the beginning of the list,
	  * and runs all the way up to index. It then swaps out (carefully)
	  * the pointers in the list.
      */
	public void add(int index, T z) {
		if(index==0) { 
			// New head node equal data (z) and pointer to old head node
			this.head = new Node<T>(z, this.head);
		} else {
			// Get the node right in front of the one we are going to add
			Node<T> pointer = nodeAt(index-1);

			// Add the new node right in front
			// New pointer.next equals data (z) and old pointer.next
			pointer.next = new Node<T>(z, pointer.next);
		}
	} 

	//// Remove an index from a list.
	public void remove(int index) { 
		if( index==0 ) { 
			head = head.next;
		} else {
			// Get all the way up to the node just in front of the node we are removing
			Node<T> current = nodeAt(index-1);

			// Now blow away any reference to the next node
			current.next = current.next.next;
		}
	}


	/// Get the index of value. If value is not in the list, return -1.
	public int indexOf(T value) { 
		int index = 0;
		Node<T> current = head;
		while(current != null) { 
			if(current.data == value) {
				return index;
			}
			index++;
			current = current.next;
		}
		return -1;
	}


	/// Get the object T at index i.
    public T get(int index) { 
        return nodeAt(index).data; 
    }


	/// Get the node at index i.
    private Node<T> nodeAt(int index) {
        Node<T> pointer = head;
        for (int i = 0; i < index; i++) {
            pointer = pointer.next;
        }
        return pointer;
    }

}

class Node<T> {
    Node<T> next;
    T data;
    Node() { 
        this(null,null);
    }
    Node(T the_datum) { 
        this(the_datum, null);
    }
    Node(T the_datum, Node<T> the_next) { 
        this.next = the_next; 
        this.data = the_datum; 
    }
}

class Point {
    public int x;
    public int y;
    public Point() {
        this.x = 0;
        this.y = 0;
    }
    public Point(int x0, int y0) { 
        this.x = x0;
        this.y = y0;
    }
    public String toString() {
        String result = String.format("(%d, %d)",this.x,this.y);
        return result;
    }
}

