import random
from Queue import Queue as queue 

class BinaryTree(object):
    """
    Binary Tree object
    """

    ####################################################
    ############### Builtins ###########################

    def __init__(self):
        """Default constructor"""
        self(self,None)

    def __init__(self,root):
        """Constructor taking a node"""
        self.root = root

    def __str__(self):
        """Turn the object into a string"""
        result = ""
        result = self.recursive_print(self.root)
        return result

    def recursive_print(self,node):
        """Recursive printing of binary tree nodes"""
        result = ""
        if(node is not None):
            result += str(node.data)+"\n"
            result += self.recursive_print(node.left)
            result += self.recursive_print(node.right)
        return result

    ####################################################
    ############### Sorting ############################

    def sort(self):
        """Public sort method"""
        self.rsort_(self.root)

    def rsort_(self,node):
        """Private recursive sort method (depth-first sort)"""
        if( node is not None and node.left is not None and node.right is not None ):
            # Swap left and right if necessary
            if(node.left.data > node.right.data ):
                temp = node.left
                node.left = node.right
                node.right = temp

            # Whether we swap or not, sort children
            self.rsort_(node.left)
            self.rsort_(node.right)
        
        # Either we're done, or we didn't have anything to do
        return

    ####################################################
    ############### Searching ##########################

    # Search assumes tree is sorted

    def search_bf(self,value):
        """Breadth-first search for value in binary tree"""
        return self.search_bf_(self.root,value)

    def search_bf_(self,root_node,value):
        """Traverse binary tree using breadth-first search (queue)"""
        q = queue()
        q.put(root_node)
        while(not q.empty()):
            nxt = q.get()
            if(nxt.data==value): 
                return nxt
            if nxt.left is not None:
                q.put(nxt.left)
            if nxt.right is not None:
                q.put(nxt.right)
        return None

    def search_df(self,value):
        """Public method to search binary tree using depth-first search (recursion)"""
        return self.search_df_(self.root,value)

    def search_df_(self,node,value):
        """search binary tree using depth-first search (recursion)"""
        if(node is not None):
            if(node.data==value):
                return node
            else:
                result = self.search_df_(node.left, value) 
                if(result):
                    return result
                result = self.search_df_(node.right, value)
                if(result):
                    return result




    ####################################################
    ############### Traverse ###########################

    # Breadth-first (queue)

    def traverse_bf(self):
        """Public method to traverse binary tree using breadth-first search (queue) and apply a function f to all Nodes"""
        self.traverse_bf_(self.root)

    def traverse_bf_(self,root_node):
        """Traverse binary tree using breadth-first search (queue)"""
        q = queue()
        q.put(root_node)
        while(not q.empty()):
            nxt = q.get()
            print(nxt.data)
            if nxt.left is not None:
                q.put(nxt.left)
            if nxt.right is not None:
                q.put(nxt.right)
        return

    # Depth-first (recursive)

    def traverse_df(self):
        """Public method to traverse binary tree using depth-first search (recursion)"""
        self.traverse_df_(self.root)

    def traverse_df_(self,node):
        """Traverse binary tree using depth-first search (recursion)"""
        if(node is not None):
            print(node.data)
            self.traverse_df_(node.left)
            self.traverse_df_(node.right)
        return







class BTNode(object):

    def __init__(self,data):
        """Constructor taking a piece of data"""
        self.data = data
        self.left  = None
        self.right = None

    def __str__(self):
        """Turn the node into a string"""
        return str(self.data)


if __name__=="__main__":

    n = BTNode(0)
    n.left  = BTNode(5)
    n.right = BTNode(18)

    b = BinaryTree(n)

    b.root.left.left  = BTNode(1)
    b.root.left.right = BTNode(3)
    
    b.root.right.left  = BTNode(14)
    b.root.right.right = BTNode(16)

    print("Printing binary tree:")
    print(b)

    def f(x):
        print(x)

    print("Traversing binary tree using breadth-first search:")
    b.traverse_bf()

    print

    print("Traversing binary tree using depth-first search:")
    b.traverse_df()

    print

    print("Sorting binary tree:")
    b.sort()

    print("Searching for node 16 in the tree:")
    print(b.search_bf(16))
    print(b.search_df(16))


