from __future__ import print_function
from collections import deque

# Illustrating the zoo of container types in Python.

# General purpose build in containers:
# dict
# list 
# set 
# tuple

def use_dict():
    print("Using dict type:")
    d = {}
    d['spam'] = 'foo'
    d['eggs'] = 'bar'
    print(d)

def use_list():
    print("Using list type:")
    z = []
    z.append(5)
    z.append(10)
    z.append(15)
    print(z)

def use_set():
    print("Using set type:")
    s = set()
    for i in range(15):
        s.add(i)
    print(s)

def use_tuple():
    print("Using tuple type:")
    t = tuple((5,10,15))
    print(t)


# High performance data containers:
# deque - double ended queue
# namedtuple - named tuple arguments
# Counter - count hashable objects
# OrderedDict - retain order added
# defaultdict - call factory function to supply missing data

def use_deque():
    # Double-ended queue ("deck"):
    # Generalization of stacks and queues
    d = deque()
    for i in range(15):
        # add to the right side of the deque
        d.append(i)

    for j in range(30,45):
        # add to the left side of the deque
        d.appendleft(j)

    # pop from right hand side
    # (this pops 14, the last number added to the right side of the deque)
    print("Popping 3 elements (from right):")
    for i in range(3):
        print("Pop... %d"%(d.pop()))

    print("Popping 5 elements from left:")
    for i in range(5):
        print("Pop... %d"%(d.popleft()))

    print("Resulting deque:")
    print(d)





if __name__=="__main__":
    print("-"*40)
    print("General Purpose Built-In Containers")
    print("-"*40)
    use_dict()
    use_list()
    use_set()
    use_tuple()

    print()

    print("-"*40)
    print("Specialized High-Performance Containers")
    print("-"*40)
    use_deque()

